/* -*- C++ -*- */
/***********************************************************************************
 ***********************************************************************************
 ****
 **** UML 1.5 Meta-model Specification
 **** Based on UML Specification 1.5 final/03-03-01
 **** Author: ks.uml@kurtstephens.com,
 **** Version: $Revision: 1.10 $
 **** Date: 2003/04/17
 **** Reference: http://kurtstephens.com/uml2code
 ****
 **** Note: This document is machine-readable by UML::Import::MetaMetaModel;
 **** Please see it for language definition.
 ****
 **** This specification is incomplete.
 ****
 ***********************************************************************************
 ***********************************************************************************
 */


/*************************************************************************************
 ************************************************************************************
 ************************************************************************************
 **
 ** This description is parsed into objects in the UML::MetaMetaModel
 ** (or UML::MetaModel) package.
 **	
 ** UML::MetaMetaModel implements a subset of the UML Metamodel for the purpose
 ** of boot-strapping the UML.
 **
 ** The UML::Export classes only use a the meta-meta-model subset interfaces
 ** while exporting.
 **
 ** Thus the Export classes have double-duty:
 ** 1. Generate code for the UML Meta-model implementation,
 ** 2. Generate code for new models generated by UML editors.
 **
 ************************************************************************************
 ************************************************************************************
 ************************************************************************************
 */


/* See p. 5-3 for UML Package Dependencies. */

Model "UML 1.5 final/03-03-01"
{

Package Foundation
{

Package Data_Types 
{ 

  /* Primitives */
  Primitive Integer;
  Primitive UnlimitedInteger;
  Primitive String;
  Primitive Name;


  Enumeration AggregationKind 
  {
    none;
    aggregate;
    composite;
  }

  Enumeration Boolean
  {
    false;
    true;
  }

  Enumeration CallConcurrencyKind
  {
    sequential;
    guarded;
    concurrent;
  }

  Enumeration ChangeableKind
  {
    changeable;
    frozen;
    addOnly;
  }

  Enumeration OrderingKind
  {
    unordered;
    ordered;
  }

  Enumeration PseudostateKind
  {
    choice;
    deepHistory;
    fork;
    initial;
    join;
    junction;
    shallowHistory;
  }

  Enumeration ScopeKind
  {
    instance;
    classifier;
  }

  Enumeration VisibilityKind
  {
    public;
    protected;
    private;
    package;
  }


  Enumeration ParameterDirectionKind
  {
    in;
    out;
    inout;
    return;
  }

  Class Geometry;
  Class LocationReference;

  Class Mapping
  {
    body : String;
    ;
  }


  Class Multiplicity
  {
    @ <#> 1 , +range : MultiplicityRange 1..* ;
  }


  Class MultiplicityRange
  {
    lower : Integer;
    upper : UnlimitedInteger;
  }

  Class Expression
  {
    language : Name;
    body : String;
  }

  : Expression
  {
    Class ArgListExpression;
    Class BooleanExpression;
    Class MappingExpression;
    Class ProcedureExpression;
    Class TimeExpression;
    Class TypeExpression;
  }

} /* End Package Foundation::Data_Types */


Package Core
{
  :: Foundation::Data_Types::*;


  /* Page 2-13; Core Package - Backbone */

  Class /Element/;

  Class /ModelElement/ : Element
  {
    name            : Name;
    visibility      : VisibilityKind; /* ElementOwnership ???*/
    isSpecification : Boolean;        /* ElementOwnership ???*/
  }

  AssociationClass ElementOwnership
  {
    visibility      : VisibilityKind;
    isSpecification : Boolean;

    @ . +ownedElement : ModelElement * , <#> +namespace : Namespace 0..1;
  }

  Class /Namespace/ : ModelElement
  {
    /* @ namespace  0..1 , ownedElement : ModelElement * /* ElementOwnership ??? */;
  }

  Class /GeneralizableElement/ : ModelElement
  {
    isRoot : Boolean;
    isLeaf : Boolean;
    isAbstract : Boolean;
  }

  Class /Classifier/ : Namespace, GeneralizableElement;

  Class Parameter : ModelElement
  {
    defaultValue  : Expression;
    kind          : ParameterDirectionKind;
    
    @            *           , +type : Classifier;
    @ +parameter * {ordered} ,       : BehavioralFeature 0..1;
  }

  Class Constraint : ModelElement
  {
    body : BooleanExpression;

    @ +constraint * , +constrainedElement : ModelElement 0..1;
  }

  Class /Feature/ : ModelElement
  {
    ownerScope : ScopeKind;
    visibility : VisibilityKind;
    
    @ +feature * {ordered} , <#> +owner : Classifier 0..1;
  }

  Class /StructuralFeature/ : Feature
  {
    multiplicity : Multiplicity;
    changeability : ChangeableKind;
    targetScope : ScopeKind;
    ordering : OrderingKind;

    @ +typedFeature *, +type : Classifier;
  }

  Class Attribute : StructuralFeature
  {
    initialValue : Expression;
  }

  Class /BehavioralFeature/ : Feature
  {
    isQuery : Boolean;
  }

  : BehavioralFeature
  {
    Class Operation
    {
      concurrency : CallConcurrencyKind;
      isRoot : Boolean;
      isLeaf : Boolean;
      isAbstract : Boolean;

      /* specification : String; /* Error? see UML 1.5 Spec, pg. 2-13, 2-45 */
      
      @ +specification , : Method *;
    }

    Class Method
    {
      body : ProcedureExpression;
    }
  }

  /* Page 2-13; Core Package - Relationships */

  Class /Relationship/ : ModelElement;

  : Relationship
  {

    Class Flow
    {
      @ +sourceFlow * , +source : ModelElement *;
      @ +targetFlow * , +target : ModelElement *;
    }

    Class Generalization
    {
      @ +generalization * , +child     : GeneralizableElement 1;
      @ +specialization * , +parent    : GeneralizableElement 1;
      @ +powertypeRange * , +powertype : Classifier           0..1;
    }

  }

  Class Association : GeneralizableElement, Relationship  /* GeneralizableElement is more dominant */
  {
/*
   This simplifies a bunch of code.
    @ <#>               1 , +connection  : AssociationEnd   2..* {ordered};
*/
    @ <#> +_association 1 , +connection  : AssociationEnd   2..* {ordered};
  }

  Class AssociationClass :  Class, Association; /* Class is more dominant */

  Class AssociationEnd : ModelElement
  {
    isNavigable : Boolean;
    ordering : OrderingKind;
    aggregation : AggregationKind;
    targetScope : ScopeKind;
    multiplicity : Multiplicity;
    changeability : ChangeableKind;
    visibility : VisibilityKind;
    
    @     +association    *              , +participant   : Classifier 1;
    @     +specifiedEnd   *              , +specification : Classifier *;
    @ <#> +associationEnd 0..1           , +qualifier     : Attribute  * {ordered} ;
  }

  /* Page 2-15; Core Package - Dependencies */

  Class Dependency :Relationship
  {
    @ +supplierDependency * , +supplier : ModelElement 1..*;
    @ +clientDependency   * , +client   : ModelElement 1..*;
  }

  : Dependency
  {

    Class Binding;
    Class Usage;
    Class Permission;
    Class Abstraction
    {
      mapping : MappingExpression;
    }

  }

  /* Page 2-16: Core Package - Classifiers */

  : Classifier
  {

    Class "Class"
    {
      isActive : Boolean;
    }

    Class Interface;

    Class Node
    {
      @ +deploymentLocation *    , +deployedComponent : Component *;
    }

    Class Component
    {
      @ +implementationLocation * , +implementation : Artifact *;
      @ +container              * , +resident       : ModelElement *;
    }

    Class Artifact;
    Class DataType;

  }

  : DataType
  {

    Class "Primitive";
    Class ProgrammingLanguageDataType
    {
      expression : TypeExpression;
    }

    Class "Enumeration"
    {
      @ +enumeration , +literal : EnumerationLiteral 1..* {ordered} ;
    }

  }

  Class EnumerationLiteral : ModelElement;
 
  /* Page 2-17: Core Package - Auxililary elements */

  : Element /* Implied?? */
  {

    AssociationClass TemplateParameter
    {
      @                                     *           ,     +defaultElement : ModelElement 0..1;

      @ . +templateParameter : ModelElement * {ordered} , <#> +template       : ModelElement 0..1;
    }

    Class TemplateArgument
    {
      @           *              , +modelElement : ModelElement 1;
      @ +argument 1..* {ordered} , +binding      : Binding      1;
    }

    Class /PresentationElement/ : Element
    {
      @ +presentation  * , +subject : ModelElement *;
    }

  }

  Class Comment : ModelElement
  {
    name : String;
    @ * , +annotatedElement : ModelElement *;
  }
  
} /* End Package Foundation::Core */


Package Extension_Mechanisms
{
  :: Foundation::Core::*, Foundation::Data_Types::*;

  /* Page 2-76; Extension Mechanisms */


  Class Stereotype : GeneralizableElement
  {
    icon : Geometry;
    baseClass : Name[*];

    @ +constrainedStereotype 0..1 , +stereotypeConstraint : Constraint    *;
    @ +owner                 0..1 , +definedTag           : TagDefinition *;
    @ +stereotype            *    , +extendedElement      : ModelElement  *;
  }

  Class TagDefinition : ModelElement
  {  
    tagType : Name;
    multiplicity : Multiplicity;
    
    @ +type 1 , +typedValue : TaggedValue *;
  }

  Class TaggedValue : ModelElement
  {
    dataValue : String[*];
    @ +taggedValue  * ,                 : ModelElement 1;
    @ +referenceTag * , +referenceValue : ModelElement *;
  }

} /* End Package Foundation::Extension_Mechanisms */


} /* End Package ::Foundation */


/* Page 2-182: Model Management */

Package Model_Management
{
  :: Foundation::Core::*, Foundation::Data_Types::*;

  Class "Package" : Namespace;
  Class Subsystem : Classifier, Package /* Classifier is more dominant */
  {
    isInstantiable : Boolean;
  }

  Class "Model" : Package;

  AssociationClass ElementImport
  { 
    visiblitity : VisibilityKind;
    alias : Name;
    isSpecification : Boolean;

    @ . : Package * , +importedElement : ModelElement *;
  }
} /* END Package ::Model_Management */


/* Page 2-93: Behavioral Elements Package */

Package Behavioral_Elements
{
  Package Common_Behavior
  {
    :: Foundation::Core::*, Foundation::Data_Types::*;

    /* Page 2-94: Common Behavior - Signals */
    Class Signal : ModelElement
    {
      @ +raisedSignal * , +context   : BehavioralFeature *;
      @ +signal 1       , +reception : Reception         0..*;
    }
    Class Exception : Signal;
    Class Reception : BehavioralFeature /* from Foundation::Core */
    {
      specification : String;
      isRoot : Boolean;
      isLeaf : Boolean;
      isAbstract : Boolean;
    }

    /* Page 2-95: Common Behavior - Procedure */
    : ModelElement
    {
      Class Procedure
      {
	language : Name;
	body : String;
	isList : Boolean;

	@ +procedure 0..1 , +expression : Expression 0..*;
	@ +procedure 0..1 , +method     : Method     0..*;
      }

      /* Page 2-96: Common Behavior - Instances */
      Class AttributeLink
      {
	@       *    ,     +attribute : Attribute 1     ;
	@ +slot 0..* , <#>            : Instance  1     ;
	@       *    ,     +value     : Instance  1     ;
      }

      Class Stimulus
      {
	@      * , <> +argument       : Instance  * {ordered} ;
	@      * ,    +receiver       : Instance  1           ;
	@      * ,    +sender         : Instance  1           ;
	@      * ,    +dispatchAction : Procedure 1           ;
      }
      Class /Instance/
      {
        @                *    ,     +classifier : Classifier 1..*;
	@ +ownedInstance 0..* , <#> +owner                   0..1; 
      }
    }

    : Instance 
    {
      Class DataValue;
      Class SubsystemInstance;
      Class ComponentInstance
      {
	@ 0..1        , +resident : Instance;
        @ +resident * ,           : NodeInstance 0..1;
      }
      Class NodeInstance;
      Class Object;
    }

    /* Page 2-97: Common Behavior - Links */
    Class Link : ModelElement
    {
      @     +communicationsLink 0..1 ,                  : Stimulus      *              ;
      @     +ownedLink          *    , <#> +owner       : Instance      0..1           ;
      @ <#>                     1    ,     +connection  : LinkEnd       2..* {ordered} ;
      @                         *    ,     +association : Association   1              ;
    }
    Class LinkEnd : ModelElement
    {
      @ <#>                    0..1  , +qualifierValue  : AttributeLink *   {ordered} ;
      @     +linkEnd           *     , +instance        : Instance      1             ;
    }
    Class LinkObject : Object, Link;
  } /* END Package ::Behavioral_Elements::Common_Behavior */


  Package Collaborations
  {
    :: Behavioral_Elements::Common_Behavior::*, 
       Foundation::Core::*, 
       Foundation::Data_Types::*
       ;

    /* Page 2-113: Collaboration - Roles */
    Class Collaboration : GeneralizableElement, Namespace
    {
      @              * ,    +usedCollaboration                       *    ;
      @ <>           * ,    +constrainingElement   : ModelElement    *    ;
      @ <#>          1 ,    +/ownedElement         : ClassifierRole  1..* ;
      @ <#>          1 ,    +/ownedElement         : AssociationRole *    ;
      @ <#> +context 1 ,                           : Interaction     *    ;
      @              * ,    +representedOperation  : Operation       0..1 ; /* --- {xor} --- */
      @              * ,    +representedClassifier : Operation       0..1 ;
    }


    Class AssociationRole : Association
    {
      multiplicity : Multiplicity;
      @ <#>                          1    , +/connection : AssociationEndRole 2..*;
      @                              *    , +base        : Association        0..1; 
      @     +communicationConnection 0..1 ,              : Message            *;
    }


    Class AssociationEndRole : AssociationEnd
    {
      collaborationMultiplicity : Multiplicity;

      @    * , +base               : AssociationEnd 0..1;
      @ <> * , +avaliableQualifier : Attribute      *;
    }
    

    Class ClassifierRole : Classifier
    {
      multiplicity : Multiplicity;

      @    +/type      1 ,                     : AssociationEndRole *    ;
      @ <>             * , +availableFeature   : Feature            *    ;
      @                * , +base               : Classifier         1..* ;
      @    +playedRole * , +conformingInstance : Instance           *    ;
    }

    Class Message : Action
    {
      @ +predecessor *    , +sucessor                            * ;
/*
      @ +activator   0..1 ,                                      * ;
*/
      @              *    , +sender             : ClassifierRole 1 ;
      @              *    , +receiver           : ClassifierRole 1 ;
      @ +playedRole  *    , +conformingStimulus : Stimulus       * ;
    } 

    /* Ommission from UML Spec. 1.5 formal/03-03-01??
    ** 
    ** p. 2-112, fig. 2-18: Association between "Message" and "Action (from Common Behavior)"
    ** In Sec. 2.9 Common Behavior; none of the diagrams or text specify Class "Action".
    ** p. Index-1 cites "Action" p 2-103.
    ** p. 2-103 has no mention of "Action".  
    ** The first item in 2.9.3 is "2.9.3.1 AttributeLink".
    **
    */
    Class /Action/ 
    {
      /* WHAT IS IT'S DEFINITION? */
      ;
    }

    /* p.2-114, Fig. 2-19 Collaborations - Interactions */
     Class Interaction : ModelElement
     {
       @ <#> +interaction 1  , +message : Message 1..* ;
     }

     /* p.2-115, Fig. 2-20 Collaborations - Instances */
     Class InteractionInstanceSet : ModelElement
     {
       @                      * ,     +interaction          : Interaction              0..1 ;
       @ +interactionInstance * , <#> +context              : CollaborationInstanceSet 1    ;
       @                      * ,     +particpatingStimulus : Stimulus                 1..* ;
       /* Complete??? */
     }


     Class CollaborationInstanceSet : ModelElement
     {
       @         * , +constrainingElement  : ModelElement  *    ;
       @         * , +particpatingLink     : Link          *    ;
       @         * , +particpatingInstance : Instance      1..* ;
       @         * , +collaboration        : Collaboration 0..1 ;  
     }
   
   /* Continue specification?!?!?! */
  } /* END Package ::Behavioral_Elements::Collaborations */

  /* Continue specification!!! */
  Package Use_Cases
  {
    :: Behavioral_Elements::Common_Behavior::*, 
       Foundation::Data_Types::*,
       Foundation::Core::*
       ;

    /* p.2-130, fig.2-21   Use Cases */
    Class Actor : Classifier;
    Class UseCase : Classifier {
	@ +addition  1 ,                 : Include        * ;
        @ +base      1 , +include        : Include        * ;
	@ +extension 1 , +extended       : Extend         * ;
        @ +base      1 ,                 : Extend         * ;
        @            1 , +extensionPoint : ExtensionPoint * ;
    }

    Class UseCaseInstance : Instance;
    Class Include : Relationship;
    Class Extend : Relationship
    {
	condition : BooleanExpression;

    }
    Class ExtensionPoint : ModelElement
    {
	location : LocationReference;

	@ +extensionPoint 1..* {ordered} , : Extend * ;
    }
  } /* End Package ::Behavioral_Elements::Collaborations */

  Package State_Machines
  {
    :: Behavioral_Elements::Common_Behavior::*,
       Foundation::Core::*,
       Foundation::Data_Types::*
       ;

    /* p. 2-141, fig. 2-24.  State Machines - Events */

    Class StateMachine : ModelElement
    {
      @     +behavior   *    , <> +context      : ModelElement    0..1 ;
      @ <#>             0..1 ,    +top          : State           1 ; /* 0..1??? */
      @ <#>             0..1 ,    +transitions  : Transition      * ; /* +??? */
      @     +submachine 1    ,                  : SubmachineState * ; 
    }

    Class Guard : ModelElement
    {
      expression : BooleanExpression;
    
      @ +guard 0..1 , <#> : Transition 1 ;
    }
    Class Transition : ModelElement
    {
      @     +internalTransition *    , <#>          : State     0..1 ;
      @ <#>                     0..1 ,     +effect  : Procedure 0..1 ;
      @ <>                      0..0 ,     +trigger : Event     0..1 ;
    }
    Class /StateVertex/ : ModelElement
    {
      @ +subvertex 0..* , <#> +container : CompositeState 0..1 ;
      @ +source    1    ,     +outgoing  : Transition     * ;
      @ +target    1    ,     +incoming  : Transition     * ;
    }
    : StateVertex {
      Class State {
        @ <#> 0..1 , +entry           : Procedure 0..1 ;
        @ <#> 0..1 , +exit            : Procedure 0..1 ;
        @ <#> 0..1 , +doActivity      : Procedure 0..1 ;
	@     0..* , +deferrableEvent : Event     0..* ;
      }
      Class SynchState;
      Class PseudoState;
      Class StubState;
    }
    : State {
      Class CompositeState
      {
        ;
      }
      Class SimpleState;
      Class FinalState;
    }
    Class SubmachineState : CompositeState;
    Class /Event/
    {
      @ <#> 0..1 , +parameter : Parameter * {ordered} ; 
    }

    /* p. 2-142, fig. 2-25.  State Machines - Events */
    : Event {
      Class SignalEvent
      {
        @ +occurance * , +signal : Signal 1 ;
      }
      Class CallEvent
      {
        @ +occurance * , +operation : Operation 1 ;
      }
      Class TimeEvent
      {
        when : TimeExpression;
      }
      Class ChangeEvent
      {
        changeExpression : BooleanExpression;
      }
    }
  } /* End Package ::Behavioral_Elements::State_Machines */


  Package Activity_Graphs
  {
    :: Behavioral_Elements::Common_Behavior::*,
       Behavioral_Elements::State_Machines::*,
       Foundation::Core::*,
       Foundation::Data_Types::*
       ;

    /* p. 2-271, fig. 2-30  Activity Graphs */
    Class ActivityGraph : StateMachine
    {
      @ <#> 1 , +partition : Partition 0..* ;
    }
    Class Partition : ModelElement
    {
      @ * , +contents : ModelElement * ; 
    }
    Class SubactivityState : SubmachineState
    {
      isDynanmic : Boolean;
      dyanmicArguments : ArgListExpression;
      dynamicMultiplicity : Multiplicity;
    }
    Class ActionState : SimpleState;
    Class ObjectFlowState : SimpleState
    {
      isSynch : Boolean;
     
      @ +state *    , +parameter : Parameter  * ;
      @        *    , +type      : Classifier 1 ;
    }
    Class ClassifierInState : Classifier
    {
      @        0..* , +inState : State      1..* ;
      @        *    , +type    : Classifier 1    ;
    }
  } /* END Package Behavioral_Elements::Activity_Graphs */


  Package Actions
  {
       :: Behavioral_Elements::State_Machines::*,
          Foundation::Core::*,
          Foundation::Data_Types::*
          ;

     /* p. 2-211, fig. 2-38  Action foundation model */

    Class Action : ModelElement
    {
      isReadOnly : Boolean;
 
      @ +predecessor 1 ,     +consequent : ControlFlow 0..* ;
      @ +successor   1 ,     +antecedent : ControlFlow 0..* ;
      @ +action      1 , <#>             : Procedure   0..1 ;
    }
    Class /Pin/ : ModelElement
    {
      multiplicity : Multiplicity;
      ordering     : OrderingKind;

      @ 0..* , +type : Classifier 0..1 ;
    }
    Class OutputPin : Pin
    {
      @ +/availableOutput 0..*           ,                : Action    0..* ;
      @ +outputPin        0..* {ordered} , <#> +action    : Action    0..1 ;
      @ +argument         0..* {ordered} , <#> +procedure : Procedure 0..1 ;        
      @ +source           1              ,     +flow      : DataFlow  0..* ; 
    }
    Class InputPin : Pin
    {
      @ +/availableInput  0..*           ,                : Action    0..* ;
      @ +inputPin         0..* {ordered} , <#> +action    : Action    0..1 ;
      @ +result           0..* {ordered} , <#> +procedure : Procedure 0..1 ;
      @ +destination      1              ,     +flow      : DataFlow  1    ; 
    }
    Class ControlFlow : ModelElement
    {
        ;
    }
      Class DataFlow : ModelElement
    {
        ;
    }
    Class /PrimitiveAction/ : Action;
    Class Procedure /* : ???? */ /* in this package or somewhere else. */
    {
      language : Name;
       body : String;
      isList : Boolean;
    }

    /* p. 2-228, fig. 2-41  Composite Actions metamodel */
    Class Clause : Element;
    Class GroupAction : Action;
    Class ConditionalAction : Action;
    Class LoopAction : Action;
    Class Variable : ModelElement;

  } /* End Package ::Actions */


} /* End Model */

/* END */
