#!/usr/local/bin/perl

use 5.6.1;
use strict;
use warnings;


use lib "../lib/perl";
use lib "gen";

{
  local $" = ', '; 
  print STDERR "INC = @INC\n";
}

use File::Path;
use IO::Handle;
use IO::Pipe;


# $DB::single = 1;

my $output_dir = 'gen';
my $file2dirs_opts = '';
my $compile;
my @exporter;

while ( @ARGV ) {
  last unless $ARGV[0] =~ /^-.+/;
  local $_ = shift @ARGV;
  if ( /^-t/ ) {
    $file2dirs_opts .= "$_ ";
  }
  elsif ( /^-d/ ) {
    $output_dir = shift;
  } 
  elsif ( /^-c/ ) {
    $compile = 1;
  }
  elsif ( /^-e/ ) {
    push(@exporter, shift);
  }
}

@exporter = ( 'perl' ) unless @exporter;

for my $exporter ( @exporter ) {
  $exporter = "UML::Export::" . ucfirst($exporter) 
  unless $exporter =~ /::/;
}


my $output;
if ( grep($_ eq '-', @ARGV) ) {
  $output = *STDOUT;
} else {
  mkpath($output_dir);
  $output = new IO::Pipe;
  $output->writer("../bin/files2dirs $file2dirs_opts -d '$output_dir' -");
}


my $model = 'UML::MetaModel1';

for my $exporter ( @exporter ) {
  use UML::MetaModel;

  #$DB::single = 1;
  my $factory = UML::MetaModel->factory;
  my $model = UML::MetaModel1->model('factory' => $factory);

  print STDERR "UML::MetaModel = ", $INC{'UML/MetaModel.pm'}, "\n";

  use UML::MetaModel1;
  UML::MetaModel1->export_Model(
				'factory' => $factory,
				'model' => $model,
				'exporter' => $exporter, 
				'output' => $output,
			       );
  
}

$output->close() unless $output eq \*STDOUT;

for my $exporter ( @exporter ) {
  if ( $compile ) {
    if ( $exporter =~ /Java/ ) {
      system("cd '$output_dir' && find . -name '*.java' | xargs javac");
    }
  }
}

#$DB::single = 1;

####################################################################################

1;

####################################################################################


### Keep these comments at end of file: ks.perl@kurtstephens.com 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

